@@ -1,15 +1,16 @@
 from abc import ABC
 import datetime
 from copy import deepcopy
-from typing import List
+from typing import List, Union
 from itertools import chain
 
 import pandas as pd
 
 from magnumapi.geometry.GeometryPlot import GeometryPlot
+from magnumapi.geometry.blocks.RectangularBlock import RectangularBlock
 from magnumapi.geometry.primitives.Line import Line
 from magnumapi.geometry.primitives.Area import Area
-from magnumapi.geometry.definitions.LayerDefinition import LayerDefinition
+from magnumapi.geometry.definitions.LayerDefinition import LayerDefinition, SlottedLayerDefinition
 from magnumapi.geometry.definitions.RelativeCosThetaBlockDefinition import RelativeCosThetaBlockDefinition
 from magnumapi.geometry.definitions.AbsoluteCosThetaBlockDefinition import AbsoluteCosThetaBlockDefinition
 from magnumapi.geometry.blocks.Block import Block
@@ -25,7 +26,11 @@ class Geometry(ABC):
 
     """
 
-    def __init__(self, blocks: List[Block], layer_defs: List[LayerDefinition]) -> None:
+    def __init__(self,
+                 blocks: List[Union[AbsoluteCosThetaBlock, RectangularBlock,
+                                    HomogenizedCosThetaBlock, RelativeCosThetaBlock]],
+                 layer_defs: List[Union[LayerDefinition, SlottedLayerDefinition]]
+                 ) -> None:
         """ Constructor of a Geometry class
 
         :param blocks: a list of instances of Block class implementations (e.g., RectangularBlock, CosThetaBlock, etc.)
@@ -36,6 +41,29 @@ class Geometry(ABC):
         self.blocks = blocks
         self.layer_defs = layer_defs
 
+    def compute_surface(self):
+        """ Method computing surface of a geometry as sum of all block surfaces
+
+        :return: a surface in mm^2
+        """
+        return sum([block.compute_surface() for block in self.blocks])
+
+    def compute_surface_cu(self):
+        """ Method computing copper surface of a block as S = ns * S_strand_cu,
+        where S_strand_cu = f_cu * pi * d_strand^2 / 4
+
+        :return: a copper surface of a block in mm^2
+        """
+        return sum([block.compute_surface_cu() for block in self.blocks])
+
+    def compute_surface_nocu(self):
+        """ Method computing copper surface of a block as S = ns * S_strand_nocu,
+        where S_strand_nocu = f_nocu * pi * d_strand^2 / 4
+
+        :return: a non-copper surface of a block in mm^2
+        """
+        return sum([block.compute_surface_nocu() for block in self.blocks])
+
     def build_blocks(self) -> None:
         """ Method building all blocks for a given geometry definition
 
@@ -66,6 +94,9 @@ class Geometry(ABC):
 
         layer_defs = [layer_def.__dict__ for layer_def in self.layer_defs]
 
+        return self.init_geometry_dict(block_defs, layer_defs)
+
+    def init_geometry_dict(self, block_defs, layer_defs):
         return {'block_defs': block_defs, 'layer_defs': layer_defs}
 
     def to_df(self) -> pd.DataFrame:
@@ -78,7 +109,7 @@ class Geometry(ABC):
     def plot_blocks(self, figsize=(10, 10), is_grid=True, xlim=(0, 80), ylim=(0, 80)) -> None:
         GeometryPlot.plot_blocks(self, figsize, is_grid, xlim, ylim)
 
-    def plotly_blocks(self, figsize=(65, 65), xlim=(0, 80), ylim=(0, 80)) -> None:
+    def plotly_blocks(self, figsize=(750, 750), xlim=(0, 80), ylim=(0, 80)) -> None:
         GeometryPlot.plotly_geometry_blocks(self, figsize, xlim, ylim)
 
     def plot_bare_blocks(self, figsize=(15, 15), is_grid=True, xlim=(0, 80), ylim=(0, 80)) -> None:
@@ -98,12 +129,12 @@ class Geometry(ABC):
 
     def to_rel_geometry(self) -> "RelativeCosThetaGeometry":
         geometry = deepcopy(self)
-        if not all([isinstance(block, CosThetaBlock) for block in self.blocks]):
-            raise TypeError('Only a geometry composed of CosThetaBlocks can be converted to a relative geometry.')
-
         # check whether areas were initialize before geometry casting
         is_any_area_initialized = any([area for block in self.blocks for area in block.areas])
 
+        if not all([isinstance(block, CosThetaBlock) for block in self.blocks]):
+            raise TypeError('Only a geometry composed of CosThetaBlocks can be converted to a relative geometry.')
+
         geometry.build_blocks()
 
         blocks = []
@@ -119,11 +150,37 @@ class Geometry(ABC):
                                                     conductor_def=block.conductor_def,
                                                     block_def=block_def))
 
-        geometry_rel = RelativeCosThetaGeometry(blocks=blocks, layer_defs=geometry.layer_defs)
+        geometry_rel = self.init_geometry_instance(blocks, geometry)
         # Return to the external context of the geometry before entering
         Geometry.recover_context(is_any_area_initialized, geometry_rel)
         return geometry_rel
 
+    def init_geometry_instance(self, blocks, geometry):
+        return RelativeCosThetaGeometry(blocks=blocks, layer_defs=geometry.layer_defs)
+
+    def homogenize(self) -> "HomogenizedCosThetaGeometry":
+        """ Class method creating a homogenized cos-theta geometry from a cos-theta geometry.
+        The method assumes that all blocks are CosThetaBlock type
+
+        :param geometry: an input cos-theta geometry
+        :return: a HomogenizedCosThetaGeometry instance
+        """
+        if not all([isinstance(block, CosThetaBlock) for block in self.blocks]):
+            raise TypeError('Only a geometry composed of CosThetaBlocks can be converted to a homogenized geometry.')
+
+        blocks = []
+        for layer_index, layer_def in enumerate(self.layer_defs):
+            for block_index in layer_def.blocks:
+                index_in_blocks = self.get_index_in_blocks_for_layer_block_index(block_index)
+                block = self.blocks[index_in_blocks]
+                homo_cos_theta_block = block.homogenize()
+                blocks.append(homo_cos_theta_block)
+
+        return self.init_homogenized_geometry(blocks, self.layer_defs)
+
+    def init_homogenized_geometry(self, blocks, layer_defs):
+        return HomogenizedCosThetaGeometry(blocks, layer_defs)
+
     def get_index_in_blocks_for_layer_block_index(self, block_index: int) -> int:
         # The indexing of blocks does not have to be incremented from block to block
         # - also in a layer the block indexing is not given
@@ -232,11 +289,14 @@ class RelativeCosThetaGeometry(Geometry):
                                               block_def=abs_block_def)
             blocks.append(abs_block)
 
-        geometry_abs = Geometry(blocks=blocks, layer_defs=deepcopy(self.layer_defs))
+        geometry_abs = self.init_geometry_instance(blocks, geometry.layer_defs)
         # Return to the external context of the geometry before entering
         Geometry.recover_context(is_any_area_initialized, geometry_abs)
         return geometry_abs
 
+    def init_geometry_instance(self, blocks, layer_defs):
+        return Geometry(blocks=blocks, layer_defs=self.layer_defs)
+
     def to_rel_geometry(self) -> "RelativeCosThetaGeometry":
         return deepcopy(self)
 
@@ -253,6 +313,9 @@ class RelativeCosThetaGeometry(Geometry):
 
         layer_defs = [layer_def.__dict__ for layer_def in self.layer_defs]
 
+        return self.init_geometry_dict(block_defs, layer_defs)
+
+    def init_geometry_dict(self, block_defs, layer_defs):
         return {'block_defs': block_defs, 'layer_defs': layer_defs}
 
 
@@ -274,26 +337,5 @@ class HomogenizedCosThetaGeometry(Geometry):
     def to_block_df(self):
         raise NotImplementedError('This method is not implemented for this class')
 
-    @classmethod
-    def with_cos_theta_geometry(cls, geometry: "Geometry") -> "HomogenizedCosThetaGeometry":
-        """ Class method creating a homogenized cos-theta geometry from a cos-theta geometry.
-        The method assumes that all blocks are CosThetaBlock type
-
-        :param geometry: an input cos-theta geometry
-        :return: a HomogenizedCosThetaGeometry instance
-        """
-        if not all([isinstance(block, CosThetaBlock) for block in geometry.blocks]):
-            raise TypeError('Only a geometry composed of CosThetaBlocks can be converted to a homogenized geometry.')
-
-        blocks = []
-        for layer_index, layer_def in enumerate(geometry.layer_defs):
-            for block_index in layer_def.blocks:
-                index_in_blocks = geometry.get_index_in_blocks_for_layer_block_index(block_index)
-                block = geometry.blocks[index_in_blocks]
-                homo_cos_theta_block = block.homogenize()
-                blocks.append(homo_cos_theta_block)
-
-        return HomogenizedCosThetaGeometry(blocks=blocks, layer_defs=geometry.layer_defs)
-
     def plotly_blocks(self, figsize=(750, 750), xlim=(0, 80), ylim=(0, 80)):
         GeometryPlot.plotly_homogenized_geometry_blocks(self, figsize, xlim, ylim)
