@@ -10,21 +10,21 @@ class DesignVariable:
     def __init__(self,
                  xl: Union[int, float],
                  xu: Union[int, float],
-                 xs: Union[int, float],
                  variable: str,
+                 layer: int,
                  bcs: str) -> None:
         """ A DesignVariable constructor
 
         :param xl: lower limit of a design variable
         :param xu: upper limit of a design variable
-        :param xs: starting value of a design variable
         :param variable: name of a design variable
+        :param layer: layer index; this brakes the compatibility with ROXIE
         :param bcs: block index, range of indices (with -) or no block indication for global variables
         """
         self.xl = xl
         self.xu = xu
-        self.xs = xs
         self.variable = variable
+        self.layer = layer
         try:
             bcs = str(int(bcs))
         except ValueError:
@@ -40,12 +40,14 @@ class DesignVariable:
 
         :return: a list of strings with variable names
         """
-        if self.bcs == '' or self.bcs == 'nan':
+        if (self.bcs == '' or self.bcs == 'nan') and (self.layer=='' or self.layer=='nan'):
             return [self.variable]
+        elif self.bcs == '' or self.bcs == 'nan':
+            return ["%s:%s:" % (self.variable, self.layer)]
         elif '-' in self.bcs:
             return self._convert_range_of_blocks_into_list()
         elif self.bcs.isnumeric():
-            return ["%s:%s" % (self.bcs, self.variable)]
+            return ["%s:%s:%s" % (self.variable, int(self.layer), self.bcs)]
         else:
             raise AttributeError('The design variable has incorrect block index value: %s.' % self.bcs)
 
@@ -66,7 +68,7 @@ class DesignVariable:
         if lower > upper:
             raise AttributeError('The lower index %d is greater than the upper one %d.' % (lower, upper))
 
-        return ['%d:%s' % (block, self.variable) for block in range(lower, upper + 1)]
+        return ['%s:%s:%d' % (self.variable, self.layer, block) for block in range(lower, upper + 1)]
 
 
 class GeneticDesignVariable(DesignVariable):
@@ -77,8 +79,8 @@ class GeneticDesignVariable(DesignVariable):
     def __init__(self,
                  xl: Union[int, float],
                  xu: Union[int, float],
-                 xs: Union[int, float],
                  variable: str,
+                 layer: int,
                  bcs: str,
                  variable_type: str,
                  bits: int) -> None:
@@ -86,16 +88,15 @@ class GeneticDesignVariable(DesignVariable):
 
         :param xl: lower limit of a design variable
         :param xu: upper limit of a design variable
-        :param xs: starting value of a design variable
         :param variable: name of a design variable
         :param bcs: block index, range of indices (with -) or no block indication for global variables
         :param variable_type: variable type (int, float) used to determine the gene to value conversion type
         :param bits: the number of bits to represent a design variable
         """
         if variable_type == 'int':
-            super().__init__(int(xl), int(xu), int(xs), variable, bcs)
+            super().__init__(int(xl), int(xu), variable, layer, bcs)
         else:
-            super().__init__(float(xl), float(xu), float(xs), variable, bcs)
+            super().__init__(float(xl), float(xu), variable, layer, bcs)
         self.bits = bits
         self.variable_type = variable_type
 
@@ -110,7 +111,7 @@ class GeneticDesignVariable(DesignVariable):
         elif self.variable_type == 'int':
             gene_int = GeneticDesignVariable.convert_gene_to_int(gene)
             if gene_int > (self.xu - self.xl):
-                gene_temp = GeneticDesignVariable.convert_int_to_gene(self.xu - self.xl)
+                gene_temp = GeneticDesignVariable.convert_int_to_gene(self.xu - self.xl, self.bits)
                 return (self.bits - len(gene_temp)) * [0] + gene_temp
             else:
                 return gene
@@ -149,13 +150,17 @@ class GeneticDesignVariable(DesignVariable):
         return int(gene_chars, 2)
 
     @staticmethod
-    def convert_int_to_gene(n: int) -> list:
+    def convert_int_to_gene(n: int, nbits: int) -> list:
         """ Static method converting an integer to a gene
 
         :param n: an integer value of a gene
+        :param nbits: number of bits in binary representation
         :return: a list of bits representing a gene
         """
-        return [n >> i & 1 for i in range(n.bit_length() - 1, -1, -1)]
+        bitstring = [n >> i & 1 for i in range(n.bit_length() - 1, -1, -1)]
+        if len(bitstring) < nbits:
+            bitstring = [0] * (nbits - len(bitstring)) + bitstring
+        return bitstring
 
 
 class CockpitGeneticDesignVariable(GeneticDesignVariable):
