@@ -48,11 +48,16 @@ class Optimization(ABC):
 
             for index, individual in enumerate(self.pop):
                 print('\tIndividual:', index)
-                self.update_model(individual)
+                try:
+                    self.update_model(individual)
 
-                fom_dct = self.calculate_figures_of_merit(individual)
-                score = self.calculate_score(fom_dct)
-                scores.append(score)
+                    fom_dct = self.calculate_figures_of_merit(individual)
+                    score = self.calculate_score(fom_dct)
+                    scores.append(score)
+                except AttributeError:
+                    fom_dct = {objective_config.objective: float('nan') for objective_config in self.config.objectives}
+                    score = PENALTY
+                    scores.append(score)
 
                 # log current computation results
                 self.logger.append_to_logger(self.decode_individual(individual), fom_dct, score)
@@ -304,6 +309,13 @@ class RoxieGeneticOptimization(GeneticOptimization):
         self.is_script_executed = is_script_executed
 
     def initialize_population(self) -> List[List[int]]:
+        pop = []
+        for _ in range(self.n_pop):
+            chromosome = RoxieGeneticOptimization.generate_random_chromosome_programmable_geometry(self.design_variables)
+            pop.append(chromosome)
+        return pop
+
+    def initialize_population_old(self) -> List[List[int]]:
         pop = []
         for _ in range(self.n_pop):
             chromosome = RoxieGeneticOptimization.generate_random_chromosome(self.design_variables)
@@ -323,17 +335,59 @@ class RoxieGeneticOptimization(GeneticOptimization):
 
         return chromosome
 
+    @staticmethod
+    def generate_random_chromosome_programmable_geometry(design_variables):
+        n_layers = max([dv.layer for dv in design_variables if 'nbl_layer' in dv.variable])
+        chromosome = []
+        for layer_index in range(1, n_layers + 1):
+            # randomly select spar thickness
+            st_dv = [dv for dv in design_variables if dv.variable == 'spar_thickness' and dv.layer == layer_index][0]
+            chromosome.extend(st_dv.generate_random_gene())
+
+            # randomly select nco
+            nco_layer_dv = [dv for dv in design_variables if dv.variable == 'nco_layer' and dv.layer == layer_index][0]
+            nbl_layer_dv = [dv for dv in design_variables if dv.variable == 'nbl_layer' and dv.layer == layer_index][0]
+            nco_layer = np.random.randint(nco_layer_dv.xl, nco_layer_dv.xu + 1)
+            nbl_layer = np.random.randint(nbl_layer_dv.xl, nbl_layer_dv.xu + 1)
+            partition = RoxieGeneticOptimization.generate_number_random_partition(nco_layer, nbl_layer)
+
+            # convert partition blocks into bits
+            nco_dv = [dv for dv in design_variables if dv.variable == 'nco' and dv.layer == layer_index][0]
+
+            for partition_el in partition:
+                chromosome.extend(GeneticDesignVariable.convert_int_to_gene(partition_el, nco_dv.bits))
+
+            if len(partition) < nbl_layer_dv.xu:
+                chromosome += [0] * (nbl_layer_dv.xu - len(partition)) * nco_dv.bits
+
+            # initiate phi_r
+            for phi_r_dv in [dv for dv in design_variables if dv.variable == 'phi_r' and dv.layer == layer_index]:
+                chromosome.extend(phi_r_dv.generate_random_gene())
+
+            # initiate alpha_rad_r
+            for alp_r_dv in [dv for dv in design_variables if dv.variable == 'alpha_rad_r' and dv.layer == layer_index]:
+                chromosome.extend(alp_r_dv.generate_random_gene())
+
+        return chromosome
+
+    @staticmethod
+    def generate_number_random_partition(number, n_slices):
+        random_in_range = [np.random.randint(1, number) for _ in range(n_slices - 1)]
+        random_in_range = [0] + sorted(random_in_range) + [number]
+        partition = [t - s for s, t in zip(random_in_range, random_in_range[1:])]
+        return sorted(partition, reverse=True)
+
     def update_model(self, individual: List[int]) -> None:
         # decoded_chromosome: a chromosome decoded into a dictionary with keys corresponding to parameter names
         # (variable and block index) mapping to parameter values
-        decoded_chromosome = self.decode_individual(individual)
-        updated_block_inputs = self.update_model_parameters_targeted(decoded_chromosome).to_dict()
+        decoded_chromosome = self.decode_chromosome_programmable_geometry(individual)
+        updated_block_inputs = self.update_model_parameters_programmable_geometry(decoded_chromosome).to_dict()
 
         # write updated input to json
         with open(self.model_input_path, 'w') as file:
             json.dump(updated_block_inputs, file, sort_keys=True, indent=4)
 
-    def decode_individual(self, individual: List[int]) -> Dict:
+    def decode_individual_old(self, individual: List[int]) -> Dict:
         block_variable_value = {}
         for index, design_variable in enumerate(self.design_variables):
             # extract gene
@@ -347,7 +401,56 @@ class RoxieGeneticOptimization(GeneticOptimization):
 
         return block_variable_value
 
-    def update_model_parameters(self, decoded_chromosome: Dict) -> Dict:
+    def decode_individual(self, individual: List[int]) -> Dict:
+        return self.decode_chromosome_programmable_geometry(individual)
+
+    def decode_chromosome_programmable_geometry(self, individual: List[int]):
+        decoded = {}
+        # remove nbl_layer and nco_layer from the list of design variables
+        index_start = 0
+        n_layers = max([dv.layer for dv in self.design_variables if 'nbl_layer' in dv.variable])
+        for layer_index in range(1, n_layers + 1):
+            nbl_layer_dv = [dv for dv in self.design_variables
+                            if dv.variable == 'nbl_layer' and dv.layer == layer_index][0]
+            # decode spar thickness
+            st_dv = [dv for dv in self.design_variables
+                     if dv.variable == 'spar_thickness' and dv.layer == layer_index][0]
+            index_end = index_start + st_dv.bits
+            value = st_dv.convert_gene_to_value(individual[index_start:index_end])
+            index_start = index_end
+            variable_names = st_dv.get_variable_names()
+            decoded = {**decoded, **dict.fromkeys(variable_names, value)}
+            # decode nco
+            nco_dv = [dv for dv in self.design_variables
+                      if dv.variable == 'nco' and dv.layer == layer_index][0]
+            for block_index in range(nbl_layer_dv.xu):
+                index_end = index_start + nco_dv.bits
+                value = GeneticDesignVariable.convert_gene_to_int(individual[index_start:index_end])
+                index_start = index_end
+                variable_name = 'nco:%d:%d' % (layer_index, block_index + 1)
+                decoded[variable_name] = value
+
+            # decode phi_r
+            for phi_r_dv in [dv for dv in self.design_variables
+                             if dv.variable == 'phi_r' and dv.layer == layer_index]:
+                index_end = index_start + phi_r_dv.bits
+                value = phi_r_dv.convert_gene_to_value(individual[index_start:index_end])
+                index_start = index_end
+                variable_names = phi_r_dv.get_variable_names()
+                decoded = {**decoded, **dict.fromkeys(variable_names, value)}
+
+            # decode alpha_rad_r
+            for alp_r_dv in [dv for dv in self.design_variables
+                             if dv.variable == 'alpha_rad_r' and dv.layer == layer_index]:
+                index_end = index_start + alp_r_dv.bits
+                value = alp_r_dv.convert_gene_to_value(individual[index_start:index_end])
+                index_start = index_end
+                variable_names = alp_r_dv.get_variable_names()
+                decoded = {**decoded, **dict.fromkeys(variable_names, value)}
+
+        return decoded
+
+    def update_model_parameters(self, decoded_chromosome: Dict) -> Geometry:
         """ Method updating model parameters given as ROXIE block definitions. Typically, it is a relative block
         definition.
 
@@ -355,18 +458,32 @@ class RoxieGeneticOptimization(GeneticOptimization):
         :return: an updated list of block definitions
         """
         geometry_model = deepcopy(self.geometry)
-        blocks_def_update = geometry_model.to_rel_geometry().to_dict()
         for block_variable, value in decoded_chromosome.items():
-            if ':' in block_variable:
-                block, variable = block_variable.split(':')
-                block = int(block)
-                blocks_def_update['block_defs'][block - 1][variable] = value
+            if ':-:' in block_variable:
+                variable, layer_index, block_index = block_variable.split(':')
+                block_index = int(block_index)
+                if variable == 'current':
+                    geometry_model.blocks[block_index - 1].block_def.current = value
+            elif ':nan:' not in block_variable:
+                variable, layer_index, block_layer_index = block_variable.split(':')
+                layer_index = int(layer_index)
+                block_layer_index = int(block_layer_index)
+
+                no_block = geometry_model.layer_defs[layer_index - 1].blocks[block_layer_index - 1]
+                block_index = geometry_model.get_index_in_blocks_for_layer_block_index(no_block)
+                if variable == 'nco':
+                    geometry_model.blocks[block_index].block_def.nco = value
+                elif variable == 'alpha':
+                    geometry_model.blocks[block_index].block_def.alpha = value
+                elif variable == 'alpha_r':
+                    geometry_model.blocks[block_index].block_def.alpha_r = value
+                elif variable == 'phi':
+                    geometry_model.blocks[block_index].block_def.phi = value
+                elif variable == 'phi_r':
+                    geometry_model.blocks[block_index].block_def.phi_r = value
 
         # Remove empty blocks
-        # ToDo: Update layer definition as well
-        block_defs = blocks_def_update['block_defs']
-        blocks_def_update['block_defs'] = [block for block in block_defs if block['nco'] > 0]
-        return blocks_def_update
+        return GeometryChange.update_layer_indexing(geometry_model)
 
     def update_model_parameters_targeted(self, decoded_chromosome: Dict) -> Geometry:
         geometry_rel = deepcopy(self.geometry)
@@ -386,7 +503,30 @@ class RoxieGeneticOptimization(GeneticOptimization):
         # extract relative geometry
         geometry_rel = geometry_abs.to_rel_geometry()
 
+        # Add more modifications here
+
         # update alpha_rad_r
+        return GeometryChange.update_alpha_radial(geometry_rel, decoded_chromosome)
+
+    def update_model_parameters_programmable_geometry(self, decoded_chromosome: Dict) -> Geometry:
+        # update spar thickness
+        geometry_rel = GeometryChange.update_spar_thickness(self.geometry, decoded_chromosome)
+
+        # update nco varying blocks
+        geometry_rel = GeometryChange.update_nco_varying_blocks(geometry_rel, decoded_chromosome)
+
+        # update phi_r
+        geometry_rel = GeometryChange.update_phi_r(geometry_rel, decoded_chromosome)
+
+        # limit minimum phi_r
+        geometry_rel = GeometryChange.limit_minimum_phi_r(geometry_rel, min_length=1)
+
+        # calculate radiality
+        geometry_abs = geometry_rel.to_abs_geometry()
+        geometry_abs = GeometryChange.calculate_radial_alpha(geometry_abs)
+
+        # update alpha_phi_rad
+        geometry_rel = geometry_abs.to_rel_geometry()
         geometry_rel = GeometryChange.update_alpha_radial(geometry_rel, decoded_chromosome)
 
         return geometry_rel
