@@ -42,7 +42,7 @@ class ModelExecutor(ABC):
                                                  config_notebook.notebook_name)
             output_ipynb_file_path = os.path.join(output_subdirectory_dir,
                                                   config_notebook.notebook_folder,
-                                                  config_notebook.notebook_name.lower().replace('.ipynb', '.py'))
+                                                  config_notebook.notebook_name.lower().replace('.ipynb', '_script.py'))
             NotebookConverter.convert_notebook_to_script(input_ipynb_file_path,
                                                          config_notebook.notebook_name.lower().split('.')[0],
                                                          output_ipynb_file_path)
@@ -77,12 +77,12 @@ class ScriptModelExecutor(ModelExecutor):
         :return: a dictionary with figures of merit if the computation was successful, otherwise a dictionary with -1
         error code is returned.
         """
-        notebook_name = model_name.split('.')[0].lower()
+        script = model_name.split('.')[0].lower() + '_script'
         cwd = os.getcwd()
 
         os.chdir(model_dir)
-        run = getattr(__import__(notebook_name), 'run_' + notebook_name)
-        print('Running %s script' % notebook_name)
+        run = getattr(__import__(script), 'run_' + script)
+        print('Running %s script' % script)
         try:
             fom_model = run(**parameters_dct)
         except Exception as exception:
