@@ -56,7 +56,6 @@ def convert_bottom_header_table_to_str(block_df: pd.DataFrame, keyword: str, suf
     :param suffix: ?
     :return: a string representation of a bottom header table
     """
-    # ToDo: is the suffix needed?
     keyword_and_length_str = "%s %d" % (keyword, len(block_df))
     if block_df.empty:
         return keyword_and_length_str
@@ -65,8 +64,7 @@ def convert_bottom_header_table_to_str(block_df: pd.DataFrame, keyword: str, suf
         block_str_lines = block_str.split('\n')
         block_str_lines = block_str_lines[1:] + [block_str_lines[0]]
         block_str = (suffix + '\n').join(block_str_lines)
-        block_str = "%s\n%s" % (keyword_and_length_str, block_str)
-        return block_str
+        return "%s\n%s" % (keyword_and_length_str, block_str)
 
 
 def read_bottom_header_table(roxie_file_path: str, keyword='CABLE') -> pd.DataFrame:
@@ -76,9 +74,9 @@ def read_bottom_header_table(roxie_file_path: str, keyword='CABLE') -> pd.DataFr
     :param keyword: a table keyword
     :return: a dataframe with content of a table given by a keyword
     """
-    text_file = open(roxie_file_path, 'r')
-    text_file_lines = text_file.read().split('\n')
-    text_file.close()
+    with open(roxie_file_path, 'r') as text_file:
+        text_file_lines = text_file.read().split('\n')
+
     index_start, n_lines = find_index_start_and_length_bottom_header_table(text_file_lines, keyword)
     return extract_bottom_header_table(text_file_lines, index_start, n_lines)
 
@@ -137,3 +135,46 @@ def extract_bottom_header_table(text_file_lines: List[str], index_start: int, n_
     if comments:
         objective_table_df['Comment'] = comments
     return objective_table_df
+
+
+def read_nested_bottom_header_table(roxie_file_path: str, keyword='LAYER') -> pd.DataFrame:
+    """ Function reading a bottom header table from ROXIE .data, .cadata, .output files
+
+    :param roxie_file_path: a path to a roxie file
+    :param keyword: a table keyword
+    :return: a nested dataframe with content of a table given by a keyword
+    """
+    with open(roxie_file_path, 'r') as text_file:
+        text_file_lines = text_file.read().split('\n')
+
+    index_start, n_lines = find_index_start_and_length_bottom_header_table(text_file_lines, keyword)
+
+    return extract_nested_bottom_header_table(text_file_lines, index_start, n_lines)
+
+
+def extract_nested_bottom_header_table(text_file_lines: List[str], index_start: int, n_lines: int):
+    """ Function extracting a bottom header template from ROXIE .data and .cadata files. The method requires a start index
+    in the table as well as the number of lines the table occupies.
+
+    :param text_file_lines: an input list of lines from a ROXIE .data or .cadata file
+    :param index_start: an index where the table starts
+    :param n_lines: the number of lines occupied by the table
+    :return: a dataframe with the table
+    """
+    value_rows = text_file_lines[index_start + 1:index_start + n_lines + 1]
+    headers = text_file_lines[index_start + n_lines + 1].split()
+
+    header_values = []
+    for value_row in value_rows:
+        header_value = {}
+        values = value_row.split()
+        for index, header in enumerate(headers[:-1]):
+            header_value[header] = int(values[index])
+
+        # It is assumed that the remaining values for a list assigned to the last column
+        # Thus it starts from len(headers) - 1 until the last but one element (the last one is \ character)
+        remaining_values =values[len(headers)-1:-1]
+        header_value[headers[-1]] = [int(value) for value in remaining_values]
+        header_values.append(header_value)
+
+    return pd.DataFrame(header_values)
\ No newline at end of file
