@@ -187,7 +187,7 @@ class RoxieToolAdapter(ToolAdapter):
             roxie_force_txt = file_read.readlines()
 
         # # convert text input to a list of floats
-        scaling = (field / target_field) ** 2
+        scaling = (target_field / field) ** 2
         roxie_force = []
         for roxie_force_txt_el in roxie_force_txt:
             row_float = [float(el) for el in roxie_force_txt_el.replace('\n', '').split(' ') if el != '']
