@@ -1036,7 +1036,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         geometry = GeometryFactory.init_with_dict(block_def, layer_def, cadata)
         geometry.build_blocks()
 
-        homo_geometry = HomogenizedCosThetaGeometry.with_cos_theta_geometry(geometry)
+        homo_geometry = geometry.homogenize()
         homo_geometry.build_blocks()
         homo_geometry.plotly_blocks()
 
