@@ -1,9 +1,11 @@
+import random
 from unittest import TestCase
 from unittest.mock import patch
 
 import pandas as pd
 import numpy as np
 
+from magnumapi.optimization.DesignVariable import GeneticDesignVariable
 from magnumapi.optimization.GeneticOptimization import GeneticOptimization
 from magnumapi.optimization.constants import PENALTY
 from tests.resource_files import create_resources_file_path
@@ -179,3 +181,5 @@ class TestGeneticOptimization(TestCase):
 
         # assert
         self.assertAlmostEqual(30.311, score, places=5)
+
+
