@@ -1,7 +1,6 @@
 import unittest
 from unittest.mock import patch
 
-from magnumapi.geometry.Geometry import HomogenizedCosThetaGeometry
 from magnumapi.geometry.GeometryFactory import GeometryFactory
 from magnumapi.cadata.CableDatabase import CableDatabase
 from magnumapi.tool_adapters.ansys.AnsysInputBuilder import AnsysInputBuilder
@@ -20,7 +19,7 @@ class TestAnsysInputBuilder(unittest.TestCase):
         geometry.build_blocks()
         geometry.plot_blocks()
 
-        homo_geometry = HomogenizedCosThetaGeometry.with_cos_theta_geometry(geometry)
+        homo_geometry = geometry.homogenize()
 
         # Find number of layers
         n_layers = homo_geometry.get_number_of_layers()
