@@ -6,7 +6,7 @@ from tests.resource_files import create_resources_file_path
 
 
 class TestRoxieToolAdapter(TestCase):
-    @patch("magnumapi.tool_adapters.roxie.RoxieToolAdapter.RoxieToolAdapter.plotly_results")
+    @patch("plotly.graph_objects.Figure.show")
     def test_parse_roxie_xml_11T(self, mock_show=None):
         # arrange
         roxie_data_xml_path = create_resources_file_path('resources/geometry/roxie/11T/reference/roxieData11T.xml')
@@ -19,10 +19,28 @@ class TestRoxieToolAdapter(TestCase):
         if mock_show is not None:
             mock_show.assert_called()
 
-    def test_parse_roxie_xml_16T(self):
+    def test_parse_roxie_xml_16T_formatted(self):
+        # arrange
         roxie_data_xml_path = create_resources_file_path('resources/geometry/roxie/16T/reference/roxieData16T.xml')
 
         # assert
         with self.assertRaises(IndexError):
             RoxieToolAdapter.parse_roxie_xml(roxie_data_xml_path)
 
+    @patch("plotly.graph_objects.Figure.show")
+    def test_parse_roxie_xml_16T_raw(self, mock_show=None):
+        # arrange
+        roxie_data_xml_path = create_resources_file_path('resources/geometry/roxie/16T/roxieData.xml')
+        roxie_data_formatted_xml_path = create_resources_file_path('resources/geometry/roxie/16T/roxieData_formatted.xml')
+
+        # act
+        RoxieToolAdapter.correct_xml_file(roxie_data_formatted_xml_path, roxie_data_xml_path)
+
+        strand_data = RoxieToolAdapter.parse_roxie_xml(roxie_data_formatted_xml_path)
+        RoxieToolAdapter.plotly_results(strand_data, xlim=(0, 85), ylim=(0, 85))
+
+        # assert
+        if mock_show is not None:
+            mock_show.assert_called()
+
+
